## Heritability Simulations and Analysis with Haseman-Elston (H-E) Regression  --  Ryan Hernandez

## Run this at the command line with:
##
##   Rscript HEplay.R help
##
## 

##set default values for all parameters
GENE="APOL1" #genotype data
h2=0.5 #true heritability
minMAF=0.05 #minimum minor allele frequency to include in analysis
INDIR="." #directory for data
Kbins = 20 #Generate Kinship Matrices for each of Kbins bins
NSIMS = 25 #total number of simulations to run
CM=1; # causal model for picking causal SNPs
Nc=10; # total number of causal variants
K=1; # total number of bins for choosing causal SNPs (only for CM=2)
fT=0; # frequency threshold for defining rare variant (only for CM=3)
fR=0; # fraction of causal variants that are rare (only for CM=3)
##parameters for sampleBetas
BM=1; # model for sampling effect size for each causal varint
fB=0.05 # fixed effect size for all causal variants (only for BM=1)
PLOT=0;
args <- commandArgs(trailingOnly=T)

if(length(args) > 0){
    for(i in 1:length(args)){
        foo=strsplit(args[i], "=");
        if(foo[[1]][1] == "GENE"){
            GENE=foo[[1]][2];
        }
        else if(foo[[1]][1] == "h2"){
            h2=as.numeric(foo[[1]][2]);
        }
        else if(foo[[1]][1] == "minMAF"){
            minMAF=as.numeric(foo[[1]][2]);
        }
        else if(foo[[1]][1] == "INDIR"){
            INDIR=foo[[1]][2];
        }
        else if(foo[[1]][1] == "Kbins"){
            Kbins=as.numeric(foo[[1]][2]);
        }
        else if(foo[[1]][1] == "NSIMS"){
            NSIMS=as.numeric(foo[[1]][2]);
        }
        else if(foo[[1]][1] == "CM"){
            CM=as.numeric(foo[[1]][2]);
        }
        else if(foo[[1]][1] == "Nc"){
            Nc=as.numeric(foo[[1]][2]);
        }
        else if(foo[[1]][1] == "K"){
            K=as.numeric(foo[[1]][2]);
        }
        else if(foo[[1]][1] == "fT"){
            fT=as.numeric(foo[[1]][2]);
        }
        else if(foo[[1]][1] == "fR"){
            fR=as.numeric(foo[[1]][2]);
        }
        else if(foo[[1]][1] == "BM"){
            BM=as.numeric(foo[[1]][2]);
        }
        else if(foo[[1]][1] == "fB"){
            fB=as.numeric(foo[[1]][2]);
        }
        else if(foo[[1]][1] == "PLOT"){
            PLOT=as.numeric(foo[[1]][2]);
        }
        else{
            if(foo[[1]][1] != "help" && foo[[1]][1] != "h"){
                cat("I misunderstood the ",i,"th option: ",args[i],"\n")
            }
            cat("usage:  Rscript h2sim.R <options>\n");
            cat("Options include:\n");
            cat("\thelp (h)\n\t\tPrints out this help menu. For options below, default values are in parentheses.\n")
            cat("\tGENE=<gene>\n\t\tNote that only APOL1 is provided\n");
            cat("\th2=<h2>\n\t\tInclude any value of h2 between [0, 1]. (0.5)\n")
            cat("\tminMAF=<minMAF>\n\t\tMinimum MAF for variants included in analysis (i.e., exclude all variants with MAF < minMAF (0.05)\n")
            cat("\tINDIR=<input directory>\n\t\tInput directory for data file (.)\n")
            cat("\tKbins=<K>\n\t\tThe number of GRM bins you want to analyze (20)\n")
            cat("\tNSIMS=<NSIMS>\n\t\tTotal number of simulations to run (25)\n")
            cat("\tCM=<CM>\n\t\tModel 1-3 for choosing causal variants (1)\n")
            cat("\tNc=<Nc>\n\t\tThe number of causal variants for analysis (10)\n")
            cat("\tK=<K>\n\t\tThe number of frequency bins to choose causal variants from when CM=2 (1)\n")
            cat("\tfT=<fT>\n\t\tFrequency threshold for defining rare variants when CM=3 (0)\n")
            cat("\tfR=<fR>\n\t\tFraction of causal variants that are rare when CM=3 (0)\n")
            cat("\tBM=<BM>\n\t\tModel 1-2 for sampling effect size of causal variants (1)\n")
            cat("\tfB=<fB>\n\t\tThe effect size for all causal variants when BM=1 (0.05)\n");
            cat("\tPLOT=<0/1>\n\t\tGenerate a plot of the inferred h2 level (0)\n");
            
            cat("example:  Rscript h2sim.R h2=0.05 NSIMS=10\n")
            quit("yes");
        }
    }
}


## read genetic data into R using function scan() and collect statistics
GZIN=gzfile(paste(INDIR,"/",GENE,".txt.012.gz",sep=""),"r");
d=scan(GZIN,nlines=1)
close(GZIN);
GZIN=gzfile(paste(INDIR,"/",GENE,".txt.012.gz",sep=""),"r");
genos = matrix(scan(GZIN),byrow=T,nc=length(d))
close(GZIN)
efreqs = apply(genos, 2, mean)/2  #frequency of non-reference allele in sample
emafs = apply(cbind(efreqs, 1-efreqs), 1, min)  #minor allele frequency in sample
genos = genos[,which(emafs >= minMAF)]
efreqs = apply(genos, 2, mean)/2  #frequency of non-reference allele in sample
emafs = apply(cbind(efreqs, 1-efreqs), 1, min)  #minor allele frequency in sample
M = ncol(genos)  #number of SNPs
N = nrow(genos)  #number of individuals

##  There are a few functions in the following file that we will need.
##    Load them using the source() function:
source(paste(INDIR,"/causalModels.R",sep=""));

###  ****  ###
IBS = makeKmatrix(Kbins);            

###  ****  ###
##  Simulations!!

h2hat = matrix(,nr=NSIMS,nc=Kbins+1); #empty matrix to store results
sres = NULL # this stores the output of the regression inference procedure
REDO = 0;
for(sim in 1:NSIMS){
    ##
    ##  Step 1: sample causal SNPs from genotypes ('genos' variable above)
    ##    There are 3 different models for how genes impact phenotypes.
    ##    The function sampleCausals() will sample a set of causal SNPs that will be used for
    ##    simulating the phenotypes:
    ##    CM=1, Nc:  Randomly sample Nc causal SNPs (favors rare variants)
    ##    CM=2, Nc, K: Randomly sample Nc causal SNPs from K frequency bins (favors common)
    ##    CM=3, Nc, fT, fR: Randomly sample fR*Nc rare variants and (1-fR)Nc common variants,
    ##      where rare variants have MAF<=fT
   
    ##  Step 2: draw effect sizes (betas) for each causal variant.  Here, we only consider 2
    ##    different models for how effect sizes are drawn. The function sampleBetas() will
    ##    take in the causal variants identified from Step 1 along with some parameters and
    ##    output a set of effect sizes.
    ##
    ##  Both of these steps are written as functions in the "sourced" file causalModels.R:
    
    causals = sampleCausals(CM=CM, Nc=Nc, K=K, fT=fT, fR=fR);
    betas = sampleBetas(causals, BM=BM, fB=fB)
    
    ##  Now we will simulate phenotypes of each individual from the causal variants
    ##    generate genetic component of phenotype
    if(Nc == 0) {
        phenosg = rep(0, N)
    }
    else if(Nc == 1) {
        phenosg = genos[,causals]*betas	
    }
    else{
        phenosg = as.matrix(genos[,causals])%*%betas	
    }
    
    ##generate phenotypes with specified heritability
    if(h2 > 1e-8){
        phenos = phenosg + rnorm(N,0,sqrt(var(phenosg)*(1/h2-1)))
    }
    else{ ## h2=0, no heritability
        phenos = rnorm(N,0,1)
    }
    
    ##create phenotype product paris for HE regression.
    nphenos = scale(phenos)

    ##product of corresponding pairs of phenotypes
    pp= nphenos%*%t(nphenos)
    pp = pp[upper.tri(pp)]
    
    ##H-E regression wooohooo!  : put a try catch here in case of error
    sres = try(cbind(sres,summary(lm(pp~IBS))$coef[2:(Kbins+1),1]));
    if(class(sres) == "try-error"){
        if(REDO>0){
            REDO = 0;
            next;
        }
        REDO = REDO+1;
        sim = sim-1;
        next;
    }
    
    h2hat[sim,] = c(sum(sres[,sim]),sres[,sim]);

    cat(h2hat[sim,1]," (mean = ",mean(h2hat[,1],na.rm=T),")\n",sep="")
}

if(PLOT){
    d=density(h2hat[,1])
    pdf(paste("h2hat_h2=",h2,"_minMAF=",minMAF,"_Kbins=",Kbins,"_CM=",CM,"_Nc=",Nc,"_K=",K,"_fT=",fT,"_fR=",fR,"_BM=",BM,"_fB=",fB,".pdf",sep=""),height=4,width=6,pointsize=12);
    par(mar=c(4,4,1,1))
    plot(d,xlab=expression(hat(h)^2),main="")
    abline(v=h2,lty=2,col='grey')
    dev.off()
}

cat("True h2 =",h2,"\nmean(estimated h2) +- 2SE =",mean(h2hat[,1]),"+-",2*sd(h2hat[,1])/sqrt(NSIMS),"\nRelative Bias =",(mean((h2hat[,1]-h2)/h2)),"\n")
