## Function that takes genotype matrix and returns Kinship matrix
makeKmatrix = function(Kbins=0){
    if(missing(Kbins)){
        cat("To generate genotype correlations, please include # bins to analyse (Kbins)\n");
    }
    else{
        ##  normalize genotypes to mean 0 and variance 1
        sefreqs = sort(emafs, index.return=T)  #variants sorted by MAF for binning
        zgenos = scale(genos)
        Ks = list() #this will store the Kinship matrix
        IBS = NULL #elements from Ks that we need for HE regression
        
        st = 1; #indicator
        for (bin in 1:Kbins){ # loop over each bin
            ## identify set of SNPs in this bin based on their frequency
            en = st+floor((M-st+1)/(Kbins-bin+1))-1
            en = max(which(sefreqs$x == sefreqs$x[en]));
            
            Mb = en-st+1 #total number of SNPs in this bin
            
            ## The kinship matrix is then just the dot product of normalized genotypes!
            Ks[[bin]] = zgenos[,sefreqs$ix[st:en]]%*%t(zgenos[,sefreqs$ix[st:en]])/(Mb)
            
            ##pull elements of kinship matrix for HE regression.
            K = Ks[[bin]]
            IBS = cbind(IBS,K[upper.tri(K)])
            
            st = en+1; #update the SNP indicator for next bin
        }
        return(IBS);
    }
}

## Function to randomly sample causal variants from genotype matrix
sampleCausals = function(CM=0, Nc=0, K=0, fT=0.01, fR=0){
    CMopt = c(1,2,3);
    if(missing(CM) | !(CM %in% CMopt)){
        cat("Please enter a causal model: CM (1, 2, or 3).\n")
        cat("CM=1, Nc:  Randomly sample Nc causal SNPs (favors rare variants)\n")
        cat("CM=2, Nc, K: Randomly sample Nc causal SNPs from K frequency bins \n")
        cat("\t(favors common variants)\n");
        cat("CM=3, Nc, fT, fR: Randomly sample fR*Nc rare variants and (1-fR)Nc common\n");
        cat("\tvariants, where rare variants have MAF<=fT\n")
    }
    if (CM == 1) { #randomly sample Nc SNPs from all SNPs in 'genos'
        if(missing(Nc)){
            cat("For CM=1, please specify Nc, the number of causal variants. For example:\n")
            cat("sampleCausals(CM=1, Nc=10)\n")
            return();
        }
        causals = sample(1:M,Nc)
        return(sort(causals));
    }
    else if (CM == 2) { #randomly sample
        if(missing(Nc) | missing(K)){
            cat("For CM=2, please specify Nc and K, the number of causal variants\n")
            cat("and the number of frequency bins to sample from.  For example:\n")
            cat("sampleCausals(CM=2, Nc=10, K=2)\n");
            return();
        }
        cbins = sample(1:K,Nc,replace=T) #randomly distribute causals into different bins
        causals = array(,dim=0) #start with empty vector
        for(i in 1:K){
            new=sample(which(emafs>=minMAF+(i-1)*(0.5-minMAF)/K*0.5 & emafs<minMAF+i*(0.5-minMAF)/K*0.5),length(which(cbins==i)));
            causals = c(causals, new)
        }
        return(sort(causals));
    }
    else if(CM == 3) {
        if(missing(Nc) | missing(fT) | missing(fR)){
            cat("For CM=3, please specify Nc (number of causal variants),\n")
            cat("fT (the frequency definition of a rare variant),\n")
            cat("and fR (the fraction of causal variants that are rare).  Example:\n")
            cat("sampleCausals(CM=3, fT=0.01, fR=0.5)\n")
            return();
        }
        set1 = which(emafs > 0 & emafs <= fT)
        set2 = which(emafs < 1 & emafs > fT)
        if(length(set1) <= round(fR*Nc) |
           length(set2) <= Nc-round(fR*Nc)){
            cat("not enough SNPs to sample that many causal variants.\n")
            cat("Please decrease Nc to at most ",
                floor(min(length(set1)/fR, length(set2)/(1-fR))),"...\n");
            SKIP = 1;
            break;
        }
        causals = c(sample(set1, round(fR*Nc)),
            sample(set2, Nc-round(fR*Nc)))
        return(sort(causals));
    }
}


## function that simulates effect sizes for each causal variant
sampleBetas = function(C=array(), BM=0, fB=0){
    BMopt = c(1,2);
    if(missing(C) | missing(BM) | !(BM %in% BMopt)){
        cat("Please enter array of causal variants (C) and a beta model: BM (1, 2, or 3).\n")
        cat("BM=1, fB: Assign all betas the same effect size, fB\n")
        cat("BM=2: Assign effect sizes to be -0.4*log10(MAF)\n")
    }
    if(BM==1){
        if(missing(fB)){
            cat("For BM=1, please specify fB (the fixed beta value to assign all causals.\n")
            cat("For example:\n")
            cat("sampleBetas(causals, BM=1, fB=0.05)\n")
        }
        else{
            betas = rep(fB, length(C))
            return(betas)
        }
    }
    else if(BM==2){
        betas =  -0.4*log(emafs[C])
        return(betas)
    }
}
